import { HttpService } from '@nestjs/axios';
import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { RESTResponseBody, RESTError } from '../models/dtos/wrappers/rest-wrapper.class';
import { firstValueFrom } from 'rxjs';
import { IDictionary } from '../models/interfaces/dictionary.interface';

@Injectable()
export class DictionariesService {
    //#region Private Members
    private _baseUrl: string;
    //#endregion

    //#region Constructor
    constructor(private readonly _httpService: HttpService, private readonly _configService: ConfigService) {
        this._baseUrl = this._configService.getOrThrow('DL_DICTIONARIES__REST__BASE_URL');
    }
    //#endregion

    //#region Public API
    public async fetchById(id: string) {
        try {
            const endpoint = `${this._baseUrl}/v1/dictionaries/by-id/${id}`;
            const response$ = this._httpService.get<RESTResponseBody<IDictionary>>(endpoint);
            const response = await firstValueFrom(response$);

            if (response.data.errors) {
                throw new RESTError('dictionaries_api_error', response.data.errors);
            }
            if (!response.data || !response.data.result) {
                throw new RESTError('dictionaries_api_error', 'empty response');
            }
            return response.data.result.data;
        } catch (e) {
            throw RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }

    public async fetchByKeyword(keyword: string) {
        try {
            const endpoint = `${this._baseUrl}/v1/dictionaries/by-keyword/${keyword}`;
            const response$ = this._httpService.get<RESTResponseBody<IDictionary>>(endpoint);
            const response = await firstValueFrom(response$);

            if (response.data.errors) {
                throw new RESTError('dictionaries_api_error', response.data.errors);
            }
            if (!response.data || !response.data.result) {
                throw new RESTError('dictionaries_api_error', 'empty response');
            }
            return response.data.result.data;
        } catch (e) {
            throw RESTError.createFromCatch(e, { keyword: 'unknown', debug: e });
        }
    }
    //#endregion
}
